/*       -*- C++ -*- */
/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/pdfplot.h $
 * $Id: pdfplot.h 15 2010-06-25 19:53:27Z schlemme $
 * $Date: 2010-06-25 21:53:27 +0200 (Fr, 25. Jun 2010) $
 * $Revision: 15 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * PDF Plotting class. 
 *
 * \defgroup cairoplotter Cairo plotting classes
 * This module contains all classes, needed for output using cairo.
 *
 *\{
 ********************************************************************
 */	  
#ifndef PDFPLOT_H
#define PDFPLOT_H
#include <limits>
#include "cexfile.h"
#include <cairomm/context.h>
#include <cairomm/surface.h>
#include "textpane.h"

/// PDF plotting class
/** This class is used to plot the diagrams into a multipage PDF file */
class pdfplot {
protected:
  struct default_value_type {
	double line_width;
	Cairo::LineCap line_cap;
	bool multi_file;
  } ;
  static default_value_type default_values;
	bool multi_file_mode; ///< true if we create one file per diagram
	int page_number; ///< page number of currently printed page.
  cexfile::rect dimensions; ///< Bounding box in cex dimension */
  cexfile::point size; ///< Size of the bounding box in cex dimension */

  /// maximum entities for concept calculation
  /** This variable is used to calculate the radius of the concept circles.
      It gives the maximum of the counting entities, which will result in 
      the maximal node radius.
      \note
      It must be converted to double, so we save it here in that type. */
  double concept_maxcount;

  /// PDF Surface from Cairo library
  Cairo::RefPtr<Cairo::Surface> surface;
  /// Cairo device context.
  Cairo::RefPtr<Cairo::Context> context;

	/// Basename for file name calculations.
  std::string filebase;

  /// Cex file object hierarchy.
  cexfile::cexfile *file;

  /// label set.
  textpane_s textlabels;
public:
  /// standard constructor
  /** This constructor is normally used to create a pdfplot object.

      \param f pointer to a parsed cex file 

      \param base base file
      name. To this string the extension .pdf will be added before the
      output file is opened.  
  */
  pdfplot(cexfile::cexfile *f,std::string base):file(f), 
						dimensions(cexfile::point(0,0),
							   cexfile::point(0,0)),
						filebase(base),
						size(0,0)
  {
    dimensions = 
      cexfile::rect(cexfile::point(std::numeric_limits<double>::infinity(),
				   std::numeric_limits<double>::infinity()), 
		    cexfile::point(-std::numeric_limits<double>::infinity(),
				   -std::numeric_limits<double>::infinity()));
	  multi_file_mode = default_values.multi_file;
	  page_number = 0;
  }

  /// get device context coordinates from cex coordinats
  /** This function provides coordinates used in the device context.
      \param p point in cex file coordinates
      \return point in device context coordinates
      \deprecated now use Cairo does that. */
  cexfile::point get_pos(const cexfile::point & p) {
    //    cexfile::point retval = p - dimensions.get_start();
    cexfile::point retval = p;
    return retval;
  }

  /// plot everything to file
  /** This function opens the PDF file and plots all diagrams in
      the cex data structure each on its own page. After that it 
      will close the file.
   */
  void plot();

  /// plot all diagrams of a lattice
  /** This funcion plots all diagrams of a given lattice.
      \param l Lattice to be plotted 
      \note The device context must be already initialized.
      \see #plot()
  */
  void plot_lattice(const cexfile::lattice &l);

  /// plot a diagram
  /** This funcion plots a diagrams.
      \param d Diagram to be plotted 
      \note The device context must be already initialized.
      \see #plot()
  */
  void plot_diagram(const cexfile::diagram &d);

  /// plot all hasse edges of a diagram.
  /** This funcion plots the hasse edges of the diagram.
      \param e set of edges to be plotted 
      \note The device context must be already initialized.
      \see #plot()
  */
  void plot_edges(const cexfile::edge_s &e);

  /// plot all label edges of a given label set.
  /** This funcion plots the label edges a given label set.
      \param s set of labels, whose edges are to be plotted 
      \note The device context must be already initialized.
      \see #plot()
  */
  void plot_label_edges(const cexfile::label_s &s);

  /// plot all concepts of a diagram.
  /** This funcion plots the concepts of the diagram.
      \param e set of concepts to be plotted 
      \param d diagram, which contains the concepts
      \note The device context must be already initialized.
      \see #plot()
  */
  void plot_concepts(const cexfile::concept_s &e, 
		     const cexfile::diagram &d);

  /// calculate a concept radius
  /** This funcion calculates the radius of a concept node in the diagram.
      \param c concept for which the radius is calculated 
      \param d diagram, which contains the concept
  */
  double get_concept_radius(const cexfile::concept &c, 
			    const cexfile::diagram &d);

  /// Calculate edge bounding box
  /** This function updates the global bounding box to 
      include all Hasse edges.
      \param e set of edges */
  void get_edge_dimensions(const cexfile::edge_s &e);
  
  /// Calculate concept bounding box
  /** This function updates the global bounding box to include
      all concept nodes.
      \param e set of concepts to be plotted 
      \param d diagram, which contains the concepts
  */      
  void get_concept_dimensions(const cexfile::concept_s &e, 
			      const cexfile::diagram &d);

  /// Calculate up label bounding boxes
  /** This function updates the global bounding box to include
      all concept nodes.
      \param d diagram, which contains the labels
  */      
  void get_uplabel_dimensions(const cexfile::diagram &d);

  /// Calculate down label bounding boxes
  /** This function updates the global bounding box to include
      all concept nodes.
      \param d diagram, which contains the labels
  */      
  void get_downlabel_dimensions(const cexfile::diagram &d);

  /// Calculate label bounding boxes
  /** This function updates the global bounding box to include
      all concept nodes.
      \param d diagram, which contains the labels
  */      
  void get_label_dimensions(const cexfile::diagram &d);

  /// plot multilabels
  /** This function plots the labels which are prepared by
      #get_label_dimensions() 

      \note The device context must be already initialized.  
      
      \see #plot()
   */
   void plot_labels();

  /// updates global bounding box to include a point or circle.
  /** This function is used to update the global bounding box to include
      a given point and circle.
      \param p Point to b included in the surface
      \param radius Distance between \c p and the border (circle radius)
  */
  void update_dimensions(const cexfile::point & p, double radius = 0) {
    if (p.get_x() - radius < dimensions.get_start().get_x())
      dimensions.get_start().set_x (p.get_x() - radius);
    else if (dimensions.get_end().get_x() < p.get_x() + radius)
      dimensions.get_end().set_x (p.get_x() + radius);
    if (p.get_y() - radius < dimensions.get_start().get_y())
      dimensions.get_start().set_y (p.get_y() - radius);
    else if (dimensions.get_end().get_y() < p.get_y() + radius)
      dimensions.get_end().set_y(p.get_y() + radius);
  }

	/// reset dimensions for a new page.
	void reset_dimensions() {
		dimensions = 
		cexfile::rect(cexfile::point(std::numeric_limits<double>::infinity(),
					     std::numeric_limits<double>::infinity()), 
			      cexfile::point(-std::numeric_limits<double>::infinity(),
					     -std::numeric_limits<double>::infinity()));
		size = cexfile::point(0,0);
	}
	
  /// set default line width from comand line
  static void set_default_line_width (std::string s) 
  {
    std::istringstream(s) >> default_values.line_width;
  }
  /// set multi file mode
  static void set_default_multi_file(bool mode = true)
	{
		default_values.multi_file = mode;
	}
protected:
	virtual bool create_surface(std::string name, double width=500, double height=500);
};

#endif /* PDFPLOT_H */
/** \} */
